
#version 310 es
#pragma only_renderers glsl31 metal
#pragma input cs_in
#pragma end

#pragma output cs_out
#pragma end

#pragma cs cs_in cs_out
precision highp float;
precision mediump sampler2D;
precision highp sampler3D;
struct BuiltinParameters
{
    float delta;
    uint dispatchWidth;
    uint nbSpawned;
    uint spawnId;
    uint systemSeed;
};
    
layout(binding = 0, std140) uniform builtiParameters
{
    BuiltinParameters builtiParametersValue;
};
struct deadListIn_type {
    uint[1] value;
};

layout(std430, binding = 0) buffer attributeBuffer {
    uint attributeBuffer_buf[];
};
    
layout(std430, binding = 1) buffer deadListIn {
    uint deadList_andCounter_Buf[];
};
#define VFX_PI 3.1415926538
#define VFX_PI2 6.28318530718
#define VFX_M_PI_F 3.14159265359
#define VFX_HALF_PI 1.57079632679
#define VFX_ONE_THIRD_PI 1.0471975512
#define VFX_FLT_MIN 1.175494351e-38
#define VFX_EPSILON 1e-5
uint wang_hash(uint seed)
{
    seed = (seed ^ 61u) ^ (seed >> 16u);
    seed *= 9u;
    seed = seed ^ (seed >> 4u);
    seed *= 0x27d4eb2du;
    seed = seed ^ (seed >> 15u);
    return seed;
}

uint AnotherHash(uint seed)
{
    seed = ((seed >> 16u) ^ seed) * 0x45d9f3bu;
    seed = ((seed >> 16u) ^ seed) * 0x45d9f3bu;
    seed = (seed >> 16u) ^ seed;
    return seed;
}

float Rand(inout uint seed)
{
    seed = 0x0019660du * seed + 0x3c6ef35fu;
    float t = uintBitsToFloat((seed >> 9u) | 0x3f800000u);
    return t - 1.0f;
}
    
vec2 Rand2(inout uint seed)
{
    return vec2(Rand(seed), Rand(seed));
}

vec3 Rand3(inout uint seed)
{
    return vec3(Rand(seed), Rand(seed), Rand(seed));
}

float random(inout uint seed, float A, float B)
{
    float rand = Rand(seed);
    return mix(A, B, rand);
}

vec3 random3(inout uint seed, vec3 A, vec3 B)
{
    vec3 rand3 = Rand3(seed);
    return mix(A, B, rand3);
}

vec3 VFXSafeNormalize(vec3 v)
{
    float sqrLength = max(VFX_FLT_MIN,dot(v,v));
    return v * inversesqrt(sqrLength);
}

vec3 HUEtoRGB(float H)
{
    float R = abs(H * 6.0 - 3.0) - 1.0;
    float G = 2.0 - abs(H * 6.0 - 2.0);
    float B = 2.0 - abs(H * 6.0 - 4.0);
    return clamp(vec3(R,G,B), 0.0, 1.0);
}

vec3 RGBtoHCV(vec3 RGB)
{
    vec4 P = (RGB.g < RGB.b) ? vec4(RGB.bg, -1.0, 2.0/3.0) : vec4(RGB.gb, 0.0, -1.0/3.0);
    vec4 Q = (RGB.r < P.x) ? vec4(P.xyw, RGB.r) : vec4(RGB.r, P.yzx);
    float C = Q.x - min(Q.w, Q.y);
    float H = abs((Q.w - Q.y) / (6.0 * C + 1e-10) + Q.z);
    return vec3(H, C, Q.x);
}

vec3 RGBtoHSV(vec3 RGB)
{
    vec3 HCV = RGBtoHCV(RGB);
    float S = HCV.y / (HCV.z + 1e-10);
    return vec3(HCV.x, S, HCV.z);
}

vec3 HSVtoRGB(vec3 HSV)
{
    return ((HUEtoRGB(HSV.x) - 1.0) * HSV.y + 1.0) * HSV.z;
}

vec3 quintic(vec3 x)
{
    return x * x * x * (x * (x * 6.0 - 15.0) + 10.0);
}

float hash(vec3 v)
{
    float HASHSCALE = 0.1031;
    v = fract(v * HASHSCALE);
    v += dot(v, v.yzx + 19.19);
    return fract((v.x + v.y) * v.z);
}

float noise01(float x, float y, float z)
{
    // Gradients
    vec3 PerlinNoise_Gradients[12];
    PerlinNoise_Gradients[0] = vec3(1, 1, 0);
    PerlinNoise_Gradients[1] = vec3(-1, 1, 0);
    PerlinNoise_Gradients[2] = vec3(1, -1, 0);
    PerlinNoise_Gradients[3] = vec3(-1, -1, 0);
    PerlinNoise_Gradients[4] = vec3(1, 0, 1);
    PerlinNoise_Gradients[6] = vec3(1, 0, -1);
    PerlinNoise_Gradients[7] = vec3(-1, 0, -1);
    PerlinNoise_Gradients[8] = vec3(0, 1, 1);
    PerlinNoise_Gradients[9] = vec3(0, -1, 1);
    PerlinNoise_Gradients[10] = vec3(0, 1, -1);
    PerlinNoise_Gradients[11] = vec3(0, -1, -1);

    vec3 p = floor(vec3(x, y, z));
    vec3 t = vec3(x, y, z) - p;
    vec3 f = quintic(t);
    vec3 v0 = p + vec3(0.0, 0.0, 0.0);
    vec3 v1 = p + vec3(1.0, 0.0, 0.0);
    vec3 v2 = p + vec3(0.0, 1.0, 0.0);
    vec3 v3 = p + vec3(1.0, 1.0, 0.0);
    vec3 v4 = v0 + vec3(0.0, 0.0, 1.0);
    vec3 v5 = v1 + vec3(0.0, 0.0, 1.0);
    vec3 v6 = v2 + vec3(0.0, 0.0, 1.0);
    vec3 v7 = v3 + vec3(0.0, 0.0, 1.0);

    int g0Index = int(hash(v0) * 12.0);
    int g1Index = int(hash(v1) * 12.0);
    int g2Index = int(hash(v2) * 12.0);
    int g3Index = int(hash(v3) * 12.0);
    int g4Index = int(hash(v4) * 12.0);
    int g5Index = int(hash(v5) * 12.0);
    int g6Index = int(hash(v6) * 12.0);
    int g7Index = int(hash(v7) * 12.0);
    vec3 g0 = PerlinNoise_Gradients[g0Index];
    vec3 g1 = PerlinNoise_Gradients[g1Index];
    vec3 g2 = PerlinNoise_Gradients[g2Index];
    vec3 g3 = PerlinNoise_Gradients[g3Index];
    vec3 g4 = PerlinNoise_Gradients[g4Index];
    vec3 g5 = PerlinNoise_Gradients[g5Index];
    vec3 g6 = PerlinNoise_Gradients[g6Index];
    vec3 g7 = PerlinNoise_Gradients[g7Index];

    return mix(
        mix(
            mix(dot(g0, t - vec3(0.0, 0.0, 0.0)), dot(g1, t - vec3(1.0, 0.0, 0.0)), f.x),
            mix(dot(g2, t - vec3(0.0, 1.0, 0.0)), dot(g3, t - vec3(1.0, 1.0, 0.0)), f.x),
            f.y
        ),
        mix(
            mix(dot(g4, t - vec3(0.0, 0.0, 1.0)), dot(g5, t - vec3(1.0, 0.0, 1.0)), f.x),
            mix(dot(g6, t - vec3(0.0, 1.0, 1.0)), dot(g7, t - vec3(1.0, 1.0, 1.0)), f.x),
            f.y
        ),
        f.z
    );
}

float fBm(int octaves, float x, float y, float z, float Fre)
{
    float amplitude = 1.0;
    float frequency = Fre;
    float result = 0.0;
    float totalAmplitude = 0.0;
    
    for (int i = 0; i < octaves; i++) {
        vec3 pos = frequency * vec3(x, y, z);
        result = result + amplitude * noise01(pos.x, pos.y, pos.z);
        totalAmplitude = totalAmplitude + amplitude;
        amplitude = amplitude / 2.0;
        frequency = frequency * 2.0;
    }
    result = result / totalAmplitude;
    return result;
}

vec3 curlNoise(vec3 oldPos, float NoiseScale, float Frequency, int Octaves)
{
    float ParticleZoneSize = 5.0;
    float halfZone = ParticleZoneSize / 2.0;
    // Get noise value
    float eps = 0.0001;
    vec3 uvy = (oldPos + vec3(halfZone, halfZone, halfZone)) / ParticleZoneSize;
    uvy = uvy * NoiseScale;

    vec3 curl = vec3(0.0, 0.0, 0.0);
    float n = 0.0, a = 0.0, b = 0.0;
    float o = fBm(Octaves, uvy.x, uvy.y, uvy.z, Frequency);

    n = fBm(Octaves, uvy.x, uvy.y - eps, uvy.z, Frequency);
    a = (o - n) / eps;
    n = fBm(Octaves, uvy.x, uvy.y, uvy.z - eps, Frequency);
    b = (o - n) / eps;
    curl.x = a - b;

    n = fBm(Octaves, uvy.x, uvy.y, uvy.z - eps, Frequency);
    a = (o - n) / eps;
    n = fBm(Octaves, uvy.x - eps, uvy.y, uvy.z, Frequency);
    b = (o - n) / eps;
    curl.y = a - b;

    n = fBm(Octaves, uvy.x - eps, uvy.y, uvy.z, Frequency);
    a = (o - n) / eps;
    n = fBm(Octaves, uvy.x, uvy.y - eps, uvy.z, Frequency);
    b = (o - n) / eps;
    curl.z = a - b;

    return curl * NoiseScale * 0.01;
}

void Orient_4(inout vec3 axisX, inout vec3 axisY, inout vec3 axisZ, mat4 viewRot) /*mode:FaceCameraPlane axes:ZY */
{
    axisX = vec3(viewRot[0].x, viewRot[1].x, viewRot[2].x);
    axisY = vec3(viewRot[0].y, viewRot[1].y, viewRot[2].y);
    axisZ = vec3(viewRot[0].z, viewRot[1].z, viewRot[2].z);
}

mat3 GetScaleMatrix(vec3 scale)
{
    return mat3(scale.x,    0.0,          0.0,
                    0.0,          scale.y,    0.0,
                    0.0,          0.0,          scale.z);
}

mat3 GetEulerMatrix(vec3 angles)
{
    vec3 s,c;
    s = sin(angles);
    c = cos(angles);

    return transpose(mat3(c.y * c.z + s.x * s.y * s.z,    c.z * s.x * s.y - c.y * s.z,    c.x * s.y,
                    c.x * s.z,                      c.x * c.z,                      -s.x,
                    -c.z * s.y + c.y * s.x * s.z,   c.y * c.z * s.x + s.y * s.z,    c.x * c.y));
}
    
mat4 GetElementToVFXMatrix(vec3 axisX,vec3 axisY,vec3 axisZ,mat3 rot,vec3 pivot,vec3 size,vec3 pos)
{
    mat3 rotAndScale = GetScaleMatrix(size);
    rotAndScale = rotAndScale * rot;
    rotAndScale = rotAndScale * transpose(mat3(axisX,axisY,axisZ));
    pos -= (pivot * rotAndScale);
    return transpose(mat4(
        vec4(rotAndScale[0],pos.x),
        vec4(rotAndScale[1],pos.y),
        vec4(rotAndScale[2],pos.z),
        vec4(0.0,0.0,0.0,1.0)));
}

mat4 GetElementToVFXMatrix(vec3 axisX,vec3 axisY,vec3 axisZ,vec3 angles,vec3 pivot,vec3 size,vec3 pos)
{
    mat3 rot = GetEulerMatrix(angles);
    return GetElementToVFXMatrix(axisX,axisY,axisZ,rot,pivot,size,pos);
}

vec2 ParticleScreenUV(vec3 position, vec3 size, vec3 pivot, vec3 angle, uint inWorldSpace, mat4 model, mat4 view, mat4 projection) {
  vec3 worldPos = (model * vec4(position, 1.0)).xyz;
  vec3 axisX = vec3(1.0, 0.0, 0.0);
  vec3 axisY = vec3(0.0, 1.0, 0.0);
  vec3 axisZ = vec3(0.0, 0.0, 1.0);

  axisX = vec3(view[0].x, view[1].x, view[2].x);
  axisY = vec3(view[0].y, view[1].y, view[2].y);
  axisZ = vec3(view[0].z, view[1].z, view[2].z);
  // Orient_4(axisX, axisY, axisZ, view);
  mat4 elementToVFX = GetElementToVFXMatrix(
                      axisX,
                      axisY,
                      axisZ,
                      angle,
                      pivot,
                      size,
                      position);
  vec3 posInWorld = position;
  if(inWorldSpace == 0u){
      posInWorld = (model * elementToVFX * vec4(0.0, 0.0, 0.0, 1.0)).xyz;
  } else {
      posInWorld = (elementToVFX * vec4(0.0, 0.0, 0.0,    1.0)).xyz;
  }
  vec4 clipPos = projection * view * vec4(posInWorld, 1.0);
  vec3 ndcPos = clipPos.xyz / clipPos.w;
  vec2 screenUV = (ndcPos.xy + vec2(1.0)) * 0.5;
  return screenUV;
}

vec3 radians3(vec3 degree)
{
    return vec3(radians(degree.x), radians(degree.y), radians(degree.z));
}


mat4 GetModelMatrix(vec3 eulerRotation, vec3 translation, float flag)
{
    vec3 s, c;
    s = sin(eulerRotation);
    c = cos(eulerRotation);
    mat4 Rx;
    Rx[0] = vec4(1.0, 0.0, 0.0, 0.0);
    Rx[1] = vec4(0.0, c.x, s.x, 0.0);
    Rx[2] = vec4(0.0, -s.x, c.x, 0.0);
    Rx[3] = vec4(0.0, 0.0, 0.0, 1.0);
    mat4 Ry;
    Ry[0] = vec4(c.y, 0.0, s.y, 0.0);
    Ry[1] = vec4(0.0, 1.0, 0.0, 0.0);
    Ry[2] = vec4(-s.y, 0.0, c.y, 0.0);
    Ry[3] = vec4(0.0, 0.0, 0.0, 1.0);
    mat4 Rz;
    Rz[0] = vec4(c.z, s.z, 0.0, 0.0);
    Rz[1] = vec4(-s.z, c.z, 0.0, 0.0);
    Rz[2] = vec4(0.0, 0.0, 1.0, 0.0);
    Rz[3] = vec4(0.0, 0.0, 0.0, 1.0);
    mat4 Rw;
    Rw[0] = vec4(1.0, 0.0, 0.0, 0.0);
    Rw[1] = vec4(0.0, 1.0, 0.0, 0.0);
    Rw[2] = vec4(0.0, 0.0, 1.0, 0.0);
    Rw[3] = vec4(-translation.x, -translation.y, -translation.z, 1.0);
    if (flag <= 0.0) return Rw * Rx * Ry * Rz;
    return Rz * Ry * Rx * Rw;
}

mat4 GetTransformInvMatrix(vec3 angles,vec3 scale,vec3 pos)
{
    mat3 invrotAndScale = GetScaleMatrix(1.0/scale);
    mat3 invrot = GetEulerMatrix(-angles);
    invrotAndScale = invrot * invrotAndScale;
    mat4 inv = mat4(
        vec4(invrotAndScale[0],0),
        vec4(invrotAndScale[1],0),
        vec4(invrotAndScale[2],0),
        vec4(0,0,0,1));
    mat4 invpos = mat4(
        vec4(1.0,0,0,-pos.x),
        vec4(0,1.0,0,-pos.y),
        vec4(0,0,1,-pos.z),
        vec4(0,0,0,1));
    return transpose(invpos * inv);
}
    
mat4 GetTransformMatrix(vec3 angles,vec3 scale,vec3 pos)
{
    mat3 rotAndScale = GetScaleMatrix(scale);
    mat3 rot = GetEulerMatrix(angles);
    rotAndScale = rotAndScale * rot;
    return transpose(mat4(
        vec4(rotAndScale[0],pos.x),
        vec4(rotAndScale[1],pos.y),
        vec4(rotAndScale[2],pos.z),
        vec4(0,0,0,1)));
}
  
layout(local_size_x = 64, local_size_y = 1, local_size_z = 1) in;

uniform sampler2D Texture2D;
uniform sampler2D Texture2D_0;
struct ExternalParameters
{
    mat4 localToWorld;
    mat4 projectionMatrix;
    mat4 viewMatrix;
    vec4 Color;
    vec4 Orientation;
    vec4 Position;
    vec4 Size;
    vec4 Size_1;
    vec4 Velocity;
    float Lifetime;
};

layout(binding = 1, std140) uniform externalParameters {
    ExternalParameters externalParametersValue;
};
void Capacity(inout vec3 position, inout vec3 size)
{

}

void SetLifetime(inout float lifetime, float Lifetime)
{
lifetime = Lifetime;
}

void PositionBox(inout vec3 position, inout vec3 direction, inout uint seed, vec4 Orientation, vec4 Position, vec4 Size)
{
vec3 localRand3 = Rand3(seed) - 0.5f;

vec3 outPosSizeGreaterThanZero = max(Size.xyz, VFX_EPSILON) * localRand3;
vec3 planeBound = 0.5f * Size.xyz;
float top    = planeBound.z - outPosSizeGreaterThanZero.z;
float bottom = planeBound.z + outPosSizeGreaterThanZero.z;
float front  = planeBound.y - outPosSizeGreaterThanZero.y;
float back   = planeBound.y + outPosSizeGreaterThanZero.y;
float right  = planeBound.x - outPosSizeGreaterThanZero.x;
float left   = planeBound.x + outPosSizeGreaterThanZero.x;

vec3 outDir = vec3(0.0,0.0,1.0);
float min = top;
if (bottom < min) { outDir = vec3(0, 0,-1);  min = bottom; }
if (front  < min) { outDir = vec3(0, 1, 0);  min = front;  }
if (back   < min) { outDir = vec3(0,-1, 0);  min = back;   }
if (right  < min) { outDir = vec3(1, 0, 0);  min = right;  }
if (left   < min) { outDir = vec3(-1,0, 0);  min = left;   }
direction = outDir;
position = Size.xyz * localRand3 + Position.xyz;

}

void SetVelocity(inout vec3 velocity, vec4 Velocity)
{
velocity.xyz = Velocity.xyz;
}

void SetColor(inout vec3 color, vec4 Color)
{
color.xyz = Color.xyz;
}

void SetSize(inout vec3 size, vec4 Size)
{
size.xyz = Size.xyz;
}


#pragma end
void compute()
    {
        uint id = gl_GlobalInvocationID.x;
        uint countLeft = deadList_andCounter_Buf[0];
        barrier();
        uint maxThreadId = builtiParametersValue.nbSpawned;
maxThreadId = min(maxThreadId, countLeft);
if(id < uint(maxThreadId) && countLeft > 0u)
{
    uint originalParticleIdx = id;
    uint particleIdx = id + uint(builtiParametersValue.spawnId);

vec3 position = vec3(0.0,0.0,0.0);
vec3 direction = vec3(1.0,0.0,0.0);
vec3 color = vec3(1.0,1.0,1.0);
vec3 velocity = vec3(0.0,0.0,0.0);
vec3 angle = vec3(0.0);
vec3 size = vec3(1.0, 1.0, 1.0);
vec3 scale = vec3(1.0, 1.0, 1.0);
float texIndex = 0.0;
float mass = 1.0;
float lifetime = 6.0;
float age = 0.0;
uint alive = 1u;
float alpha = 1.0;
vec4 customAttribute = vec4(0.0);
vec3 targetPosition = vec3(0.0,0.0,0.0);
uint eventCount = 0u;
float rateCount = 0.0;
vec3 pivot = vec3(0.0, 0.0, 0.0);
vec2 uv = vec2(0.0);
uint particleId = particleIdx;
uint seed = wang_hash(particleIdx ^ builtiParametersValue.systemSeed);
uint inWorldSpace = 0u;
mat4 localToWorld = externalParametersValue.localToWorld;
mat4 projectionMatrix = externalParametersValue.projectionMatrix;
mat4 viewMatrix = externalParametersValue.viewMatrix;
float deltatime = builtiParametersValue.delta;
{

    Capacity(position, size);
}
{

    SetLifetime(lifetime, externalParametersValue.Lifetime);
}
{

    PositionBox(position, direction, seed, externalParametersValue.Orientation, externalParametersValue.Position, externalParametersValue.Size);
}
{

    SetVelocity(velocity, externalParametersValue.Velocity);
}
{

    vec2 var_1 = ParticleScreenUV(position, size, pivot, angle, inWorldSpace, localToWorld, viewMatrix, projectionMatrix);;
    vec4 var_5 = texture(Texture2D_0, var_1);;
    float var_11 = var_5.x;
    float var_12 = var_5.y;
    float var_13 = var_5.z;
    vec3 var_16 = vec3(var_11,var_12,var_13);;
    vec3 var_20 = RGBtoHSV(var_16);;
    vec3 var_22 = max(var_20,vec3(0, 1, 1));;
    vec3 var_25 = HSVtoRGB(var_22);;
    vec3 var_27 = mix(var_25, vec3(1, 1, 1), 0.60);;
    SetColor(color, vec4(var_27, 1.0));
}
{

    float var_2 = random(seed,0.50,2.50);;
    vec3 var_7 = vec3(var_2,var_2,var_2);;
    SetSize(size, vec4(var_7, 1.0));
}

    if(alive != 0u){

uint insert_index = uint(atomicAdd(deadList_andCounter_Buf[0],-1u));
        
uint index_offset = uint(deadList_andCounter_Buf[insert_index]);
//deadList_andCounter_Buf[0] = deadList_andCounter_Buf[0] - 1u;

uint index_c = index_offset << 6;
attributeBuffer_buf[index_c     ]  = floatBitsToUint(position.x);
attributeBuffer_buf[index_c + 1u ]  = floatBitsToUint(position.y);
attributeBuffer_buf[index_c + 2u ]  = floatBitsToUint(position.z);
attributeBuffer_buf[index_c + 3u ]  = seed;

attributeBuffer_buf[index_c + 4u ]  = floatBitsToUint(velocity.x);
attributeBuffer_buf[index_c + 5u ]  = floatBitsToUint(velocity.y);
attributeBuffer_buf[index_c + 6u ]  = floatBitsToUint(velocity.z);

attributeBuffer_buf[index_c + 7u ]  = floatBitsToUint(direction.x);
attributeBuffer_buf[index_c + 8u ]  = floatBitsToUint(direction.y);
attributeBuffer_buf[index_c + 9u ]  = floatBitsToUint(direction.z);

attributeBuffer_buf[index_c + 10u ]  = floatBitsToUint(lifetime);
attributeBuffer_buf[index_c + 11u ]  = floatBitsToUint(age);
attributeBuffer_buf[index_c + 12u ]  = alive;
attributeBuffer_buf[index_c + 13u ]  = floatBitsToUint(customAttribute.x);
attributeBuffer_buf[index_c + 14u ]  = floatBitsToUint(color.x);
attributeBuffer_buf[index_c + 15u ]  = floatBitsToUint(color.y);
attributeBuffer_buf[index_c + 16u ]  = floatBitsToUint(color.z);
attributeBuffer_buf[index_c + 17u ] = floatBitsToUint(alpha);
attributeBuffer_buf[index_c + 18u ] = floatBitsToUint(customAttribute.y);
attributeBuffer_buf[index_c + 19u ] = floatBitsToUint(targetPosition.x);
attributeBuffer_buf[index_c + 20u ] = floatBitsToUint(targetPosition.y);
attributeBuffer_buf[index_c + 21u ] = floatBitsToUint(targetPosition.z);
attributeBuffer_buf[index_c + 22u ] = floatBitsToUint(size.x);
attributeBuffer_buf[index_c + 23u ] = floatBitsToUint(size.y);
attributeBuffer_buf[index_c + 24u ] = floatBitsToUint(size.z);
attributeBuffer_buf[index_c + 25u ] = eventCount;
attributeBuffer_buf[index_c + 26u ] = floatBitsToUint(rateCount);
attributeBuffer_buf[index_c + 30u ] = floatBitsToUint(customAttribute.z);
attributeBuffer_buf[index_c + 31u ] = floatBitsToUint(customAttribute.w);
attributeBuffer_buf[index_c + 32u ] = floatBitsToUint(texIndex);
attributeBuffer_buf[index_c + 33u ] = floatBitsToUint(mass);
attributeBuffer_buf[index_c + 34u ] = floatBitsToUint(pivot.x);
attributeBuffer_buf[index_c + 35u ] = floatBitsToUint(pivot.y);
attributeBuffer_buf[index_c + 36u ] = floatBitsToUint(pivot.z);
attributeBuffer_buf[index_c + 37u ] = particleId;
attributeBuffer_buf[index_c + 38u ] = floatBitsToUint(angle.x);
attributeBuffer_buf[index_c + 39u ] = floatBitsToUint(angle.y);
attributeBuffer_buf[index_c + 40u ] = floatBitsToUint(angle.z);
attributeBuffer_buf[index_c + 41u ] = floatBitsToUint(scale.x);
attributeBuffer_buf[index_c + 42u ] = floatBitsToUint(scale.y);
attributeBuffer_buf[index_c + 43u ] = floatBitsToUint(scale.z);
attributeBuffer_buf[index_c + 44u ] = inWorldSpace;
attributeBuffer_buf[index_c + 45u ] = originalParticleIdx;
}
}
return;
}
